/*
 * JGraLab - The Java Graph Laboratory
 *
 * Copyright (C) 2006-2013 Institute for Software Technology
 *                         University of Koblenz-Landau, Germany
 *                         ist@uni-koblenz.de
 *
 * For bug reports, documentation and further information, visit
 *
 *                         https://github.com/jgralab/jgralab
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses>.
 *
 * Additional permission under GNU GPL version 3 section 7
 *
 * If you modify this Program, or any covered work, by linking or combining
 * it with Eclipse (or a modified version of that program or an Eclipse
 * plugin), containing parts covered by the terms of the Eclipse Public
 * License (EPL), the licensors of this Program grant you additional
 * permission to convey the resulting work.  Corresponding Source for a
 * non-source form of such a combination shall include the source code for
 * the parts of JGraLab used as well as that of the covered work.
 */
package de.uni_koblenz.jgralab.greql.funlib.strings;

import java.util.HashMap;
import java.util.regex.Pattern;

import de.uni_koblenz.jgralab.greql.funlib.Description;
import de.uni_koblenz.jgralab.greql.funlib.Function;

public class ReMatch extends Function {
	
	@Description(params = {"s", "regex"}, description = "Returns true, iff the given string matches the given regular expression. \n"
		+ "Can be used as infix operator: myString =\\textasciitilde{} myRegexp.",
		categories = Category.STRINGS)
	public ReMatch() {
		super(50, 1, 0.1);
	}

	// cache is also used by Split function
	static HashMap<String, Pattern> patternCache = new HashMap<String, Pattern>();

	public Boolean evaluate(String s, String regex) {
		Pattern pat = patternCache.get(regex);
		if (pat == null) {
			pat = Pattern.compile(regex);
			patternCache.put(regex, pat);
		}
		return pat.matcher(s).matches();
	}
}
