/*
 * JGraLab - The Java Graph Laboratory
 *
 * Copyright (C) 2006-2013 Institute for Software Technology
 *                         University of Koblenz-Landau, Germany
 *                         ist@uni-koblenz.de
 *
 * For bug reports, documentation and further information, visit
 *
 *                         https://github.com/jgralab/jgralab
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses>.
 *
 * Additional permission under GNU GPL version 3 section 7
 *
 * If you modify this Program, or any covered work, by linking or combining
 * it with Eclipse (or a modified version of that program or an Eclipse
 * plugin), containing parts covered by the terms of the Eclipse Public
 * License (EPL), the licensors of this Program grant you additional
 * permission to convey the resulting work.  Corresponding Source for a
 * non-source form of such a combination shall include the source code for
 * the parts of JGraLab used as well as that of the covered work.
 */
package de.uni_koblenz.jgralab.utilities.gui.undo;

import javax.swing.UIManager;
import javax.swing.undo.CompoundEdit;

/**
 * {@link CompoundGraphEdit} basically is named {@link CompoundEdit}. It can be
 * used to group many simple graph edits into application specific edit units.
 * 
 * @author ist@uni-koblenz.de
 */
public class CompoundGraphEdit extends CompoundEdit {
	private static final long serialVersionUID = 1061803791604107837L;
	private String name;

	/**
	 * Creates a {@link CompoundGraphEdit} named <code>name</code>.
	 * 
	 * @param name
	 *            the name of the new {@link CompoundGraphEdit}
	 */
	public CompoundGraphEdit(String name) {
		this.name = name;
	}

	@Override
	public String getPresentationName() {
		return name;
	}

	@Override
	public String getUndoPresentationName() {
		String name = getPresentationName();
		if (!"".equals(name)) {
			name = UIManager.getString("AbstractUndoableEdit.undoText") + " "
					+ name;
		} else {
			name = UIManager.getString("AbstractUndoableEdit.undoText");
		}

		return name;
	}

	@Override
	public String getRedoPresentationName() {
		String name = getPresentationName();
		if (!"".equals(name)) {
			name = UIManager.getString("AbstractUndoableEdit.redoText") + " "
					+ name;
		} else {
			name = UIManager.getString("AbstractUndoableEdit.redoText");
		}

		return name;
	}
}
